#!/usr/local/bin/perl -w

# usage:  merge.pl file1.bib file2.bib ... > outputfile
# at end of outputfile is a list of duplicate keys

use strict;
$|++;  #turn off stout/stderr buffering

use Text::BibTeX;

# we are going to want to sort by keys
sub bykey {$a->key cmp $b->key;}

print <<USAGE unless (@ARGV);
Usage: merge.pl file1.bib file2.bib file3.bib ... > outputfile.bib

This script merges together a (memory limited) set of BibTeX bibliography
files and sorts them alphanumerically by key.  It appends a warning at the
end of the output for duplicated keys, but the user will need to remove
duplicated entries or change duplicated keys manually.

LIMITATIONS:
  1) comments get lost or at least misplaced
  2) some ignorable warnings may be dumped to the terminal
  3) this code hasn't been extensively tested, and might lose entries,
     trash files, or create a mess that makes Tempo look well-organized
     by comparison.  Use at your own risk.  If the program breaks,
     please dispose of the pieces in an environmentally sensitive way.
           --Steve Thorsett
USAGE

# we hold all entries in memory!
my @entries;

# loop through files on command line and read in entries
foreach (@ARGV) {
  my $bibfile = new Text::BibTeX::File $_;
  $bibfile->preserve_values(1);

  while (my $entry = new Text::BibTeX::Entry $bibfile) {
  warn "error in input, entry $entry" unless $entry->parse_ok;
  push @entries, $entry;
  }
}

# now write them in sorted order
foreach (sort bykey @entries) {
  $_->print();
}

# now find duplicates
my %seen = ();
foreach (@entries) {
  $seen{$_->key}++;
  next unless $seen{$_->key} > 1;
  print "Two ",$_->key," tags seen\n" if $seen{$_->key} == 2;
  print "Make that ",$_->key," tags seen\n" if $seen{$_->key} > 2;
}
